//*********************************************************************************************************************/
/* LX117B LCD Library
 * Letzte nderung:			20.03.2019
 * letzter Bearbeiter:		Eberl Florian
 * Ersteller:				Eberl Florian
 * Vorlage:					Adafruit ST7735xx Library
 * Mehrfachnutzung:			NEIN,nur 1 LCD
 * 
 * Hinweise zur Fremdlibrary:
 * Es wurden nur die Standartwerte aus der Fremdlibrary entnommen. Sonstige Routinen sind komplett verschieden 
 *
 * Wichtig:
 * Am besten den SPI mit DMA Channel nutzen. Das ist eine Einstellung, die beim Init des SPI-CHANNELS geschieht, also nicht von den Devices beeinflusst werden kann.
 * Es sollte also der SPI des Gerts genutzt werden, der eh schon mit DMA luft.
 * 
 * Optimierungen:
 * Beim full-refresh mssen zuvor der Displaybereich und der SRAM-Write Befehl gesendet werden. Das geschieht derzeit ber Polling 
 * (Funktion: _window_set) und sollte aber vollautomatisch ablaufen. Dafr muss es ber Interrupts laufen. Folgendes soll also gemacht werden:
 * Zuerst werden die Befehle in die Queue gelegt und dann die Daten wenn der full-refresh aufgerufen wird. Der stop_refresh muss also zwei Queue eintrge
 * lesen und prfen
 *
**********************************************************************************************************************/
// STATUS: V0.1 -> bekannte Optimierungen
/*********************************************************************************************************************/

#ifndef  LX177_H
#define LX177_H

	#include <stdio.h>
	#include "freertos/FreeRTOS.h"
	#include "freertos/task.h"
	#include "driver/gpio.h"
	#include "driver/spi_master.h"
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include "driver/gpio.h"
	#include "esp_log.h"
	#include "esp_heap_caps.h" 
	#include "LX_FONT.h"

//------------------------------------------------------------------------------------------------------
//							Registerwerte und Definitionen
//------------------------------------------------------------------------------------------------------
	#define ST7735_NOP  0x00
	#define ST7735_SWRESET  0x01
	#define ST7735_RDDID  0x04
	#define ST7735_RDDST  0x09
	#define ST7735_SLPIN  0x10
	#define ST7735_SLPOUT  0x11
	#define ST7735_PTLON  0x12
	#define ST7735_NORON  0x13
	#define ST7735_INVOFF  0x20
	#define ST7735_INVON  0x21
	#define ST7735_DISPOFF  0x28
	#define ST7735_DISPON  0x29
	#define ST7735_CASET  0x2A
	#define ST7735_RASET  0x2B
	#define ST7735_RAMWR  0x2C
	#define ST7735_RAMRD  0x2E
	#define ST7735_PTLAR  0x30
	#define ST7735_COLMOD  0x3A
	#define ST7735_MADCTL  0x36
	#define ST7735_FRMCTR1  0xB1
	#define ST7735_FRMCTR2  0xB2
	#define ST7735_FRMCTR3  0xB3
	#define ST7735_INVCTR  0xB4
	#define ST7735_DISSET5  0xB6
	#define ST7735_PWCTR1  0xC0
	#define ST7735_PWCTR2  0xC1
	#define ST7735_PWCTR3  0xC2
	#define ST7735_PWCTR4  0xC3
	#define ST7735_PWCTR5  0xC4
	#define ST7735_VMCTR1  0xC5
	#define ST7735_RDID1  0xDA
	#define ST7735_RDID2  0xDB
	#define ST7735_RDID3  0xDC
	#define ST7735_RDID4  0xDD
	#define ST7735_PWCTR6  0xFC
	#define ST7735_GMCTRP1  0xE0
	#define ST7735_GMCTRN1  0xE1	
	#define DELAY_FLAG      0x80   

	//Colour of Tab
	enum ST7735_DISPLAY_TYPE {
		BLUETAB_128_160,
		GREENTAB_128_160,
		REDTAB_128_160,
		BLACKTAB_128_160,
	};

	//Orientation
	enum ST7735_ORIENTATION {
		ST7735_LANDSCAPE,
		ST7735_PORTRAIT,
		ST7735_LANDSCAPE_INV,
		ST7735_PORTRAIT_INV
	};

	//Display Modes
	enum ST7735_MADCTL_ARGS {
		MADCTL_MY = 0x80, // Mirror Y
		MADCTL_MX = 0x40, // Mirrror x
		MADCTL_MV = 0x20, // Swap XY
		MADCTL_ML = 0x10, // Scan address order
		MADCTL_RGB = 0x00,
		MADCTL_BGR = 0x08,
		MADCTL_MH = 0x04 	// Horizontal scan oder
	};

//------------------------------------------------------------------------------------------------------
//							Typendefinitionen
//------------------------------------------------------------------------------------------------------
	//Typedefs
	typedef struct __attribute__((__packed__))
	{	
		uint8_t reg;
		uint8_t data[16];		
	} CMD_t;

	typedef union
	{
		uint16_t Dot[128][160];
		uint16_t Data[20480];
	}LCD_Data_t;
//------------------------------------------------------------------------------------------------------
//							Globale Variablen
//------------------------------------------------------------------------------------------------------
	static const char *LX_TAG = "GLCD:";
	extern const uint8_t st7735_blue_init[];
	extern const uint8_t st7735_red_init1[];
	extern const uint8_t st7735_red_init_green2[];
	extern const uint8_t st7735_red_init_red2[];
	extern const uint8_t st7735_red_init3[];
	extern spi_transaction_t LX_DATA;
	extern LCD_Data_t *LX_SRAM;

//------------------------------------------------------------------------------------------------------
//							InitFunktionen mit Polling
//------------------------------------------------------------------------------------------------------

	/* SPI Init
	 * (*)Handler-> Pointer auf Handler
	 *  SPI_Channel -> SPI Channel (VSPI oder HSPI)
	 *  DMA_Channel -> (0=kein DMA, 1/2=DMA Channel 1/2)*/
	void LX_Init_SPI(spi_host_device_t SPI_Channel, gpio_num_t Mosi, gpio_num_t Clk, int DMA_Channel);

	/* Chip Init (internes Polling)*/
	void LX_Init_Chip(spi_device_handle_t *SPI_Handler, enum ST7735_DISPLAY_TYPE Colour_of_tab);

	/* Einzelnen Befehl mithilfe von Polling setzen*/
	void LX_Write_Command(spi_device_handle_t *SPI_Handler, uint8_t *Data, uint8_t Bytes_to_send);

	/* Device auf dem SPI Bus erstellen*/
	void LX_Create_Device(spi_host_device_t SPI, spi_device_handle_t *SPI_Handler, gpio_num_t NSEL, gpio_num_t A0_Pin);

	void ST7735_write_Commandset(spi_device_handle_t *SPI_Handler, const uint8_t *Adress);

	void LX_set_orientation(spi_device_handle_t *SPI_Handler, enum ST7735_ORIENTATION orientation, enum ST7735_DISPLAY_TYPE Colour_of_tab);

	void ST7735_window_set(spi_device_handle_t *SPI_Handler, uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);

	//------------------------------------------------------------------------------------------------------
	//								  SRAM Manipulationen
	//------------------------------------------------------------------------------------------------------
	void LX_delete_SRAM();
	void LX_draw_rect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint16_t Colour, uint8_t Line_width);
	void LX_draw_hline(uint8_t y1, uint8_t y2, uint8_t x, uint16_t Colour, uint8_t Line_width);
	void LX_draw_vline(uint8_t x1, uint8_t x2, uint8_t y, uint16_t Colour, uint8_t Line_width);
	void LX_draw_char(int16_t x, int16_t y, const GFXglyph *glyph, const GFXfont *font, uint8_t size, uint16_t color);
	void LX_draw_text(int8_t x, int8_t y, char *text, const GFXfont *p_font, uint8_t size, uint16_t color);
	void LX_fill_rect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2, uint16_t Colour);

	//------------------------------------------------------------------------------------------------------
	//							Sendefunktionen mit Interrupt
	//------------------------------------------------------------------------------------------------------


	/*Daten aus der Arbeitsvariable an das Display senden. Die Daten werden in die Queue gelegt
	 *und mssen spter unbedingt mit der stop_refresh-Funktion abgeholt werden*/
	void LX_start_full_refresh(spi_device_handle_t *SPI_Handler);

	/*Die Queue abfragen. Sollte der falsche Queue-Eintrag erwischt werden gibt es einen Assert-Fehler
	 *Folglich strzt der Controller ab bei einem solchen Fehler*/
	void LX_stop_full_refresh(spi_device_handle_t *SPI_Handler);

#endif
