// TFT Test for LX177B Library
#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "esp_system.h"
#include "esp_wifi.h"
#include "esp_event_loop.h"
#include "esp_log.h"
#include "esp_attr.h"
#include "nvs_flash.h"
#include "driver/gpio.h"
#include "sdkconfig.h"
#include "LX177B.h"
#include "freertos/queue.h"


//LCD Grundvariablen
TickType_t xDelay30ms = pdMS_TO_TICKS(30);


//Struct fr LCD
typedef struct
{
	uint8_t Life[4];	
	uint8_t Mode;
	uint8_t Modeval;
}LCD_Trans_t;
 
//Queue fr LCD (4 Eintrge a 6 bytes)
QueueHandle_t  LCD_Queue_handler;

void lcd_task(void *pvParameter)
{
	//SPI Handler
	spi_device_handle_t Display_Handler[4];
	
	const uint16_t Col_a = 0b1111101000110010;
	const uint16_t Col_b = 0x291F;
	const uint16_t Col_white = 0xFFFF;
	const uint16_t Col_red =	0b1111100000000000;
	const uint16_t Col_orange =	0b1111100110000000;
	const uint16_t Col_blue = 0b0000001111100000;

	//Display Data
	LCD_Trans_t LCD_Data_REC;
	
	//Display Pos
	const uint8_t Life_y_Pos = 80;
	const uint8_t Boost_y_Pos = 100;
	
	
	
	//################ INIT #################/
	// Hardware kann nur 3 CS bedienen, daher 2 SPI Busse fr 4 Gerte :(
	// Mosi=4 , SCK=14 , A0 = 5 , CS1=2 , CS2= 26, CS3= 25, CS4= 32
	// Mosi=25 , SCK=27, CS4= 18
	LX_Init_SPI(VSPI_HOST, 4, 14, 1);	
	//LX_Init_SPI(VSPI_HOST, 23, 24, 2);	
		
	LX_Create_Device(VSPI_HOST, &Display_Handler[0], 2, 5);		//pink
	LX_Create_Device(VSPI_HOST, &Display_Handler[1], 26, 5);	//gelb
	LX_Create_Device(VSPI_HOST, &Display_Handler[2], 25, 5);	//grn
	//LX_Create_Device(VSPI_HOST, &Display_Handler[3], 32, 5);	//rot
	
	LX_Init_Chip(&Display_Handler[0], BLACKTAB_128_160);
	LX_Init_Chip(&Display_Handler[1], BLACKTAB_128_160);
	LX_Init_Chip(&Display_Handler[2], BLACKTAB_128_160);
	//LX_Init_Chip(&Display_Handler[3], BLACKTAB_128_160);
	
	LX_set_orientation(&Display_Handler[0], ST7735_LANDSCAPE, BLACKTAB_128_160);
	LX_set_orientation(&Display_Handler[1], ST7735_LANDSCAPE, BLACKTAB_128_160);
	LX_set_orientation(&Display_Handler[2], ST7735_LANDSCAPE, BLACKTAB_128_160);
	//LX_set_orientation(&Display_Handler[3], ST7735_LANDSCAPE, BLACKTAB_128_160);
	
	
	while (1)
	{	
		//Neue Daten abfragen
		xQueueReceive(LCD_Queue_handler, &LCD_Data_REC, NULL);
		
		
		for (uint8_t z = 0; z < 3; z++)
		{
			//LCD SRAM lschen
			LX_delete_SRAM();		
		
			switch (LCD_Data_REC.Mode)
			{
				//Mode=1 -> Balkenanzeige
				case 1: 
					LX_draw_rect(0, 0, 159, 30, Col_b, 2);
					LX_draw_text(30, 20, "Boostmode", &FreeSans, 1, Col_a);	
			
					LX_draw_text(2, (Life_y_Pos + 12), "Leben", &TomThumb, 2, Col_white);
					LX_draw_rect(51, Life_y_Pos, 159, (Life_y_Pos + 15), Col_white, 2);
					for (int i = 1; i <= LCD_Data_REC.Life[z]; i++)
					{
						LX_fill_rect((56 + (i - 1)*(30 + 4)), (Life_y_Pos + 3), (56 + 30*i + 4*(i - 1)), (Life_y_Pos + 12), Col_red);			
					}
			
					LX_draw_text(2, (Boost_y_Pos + 12), "Boost", &TomThumb, 2, Col_white);
					LX_draw_rect(51, Boost_y_Pos, 159, (Boost_y_Pos + 15), Col_white, 2);
					LX_fill_rect(56, (Boost_y_Pos + 3), (56 + LCD_Data_REC.Modeval), (Boost_y_Pos + 12), Col_blue);	
			
				break;
	
				case 2:
					LX_draw_rect(0, 0, 159, 30, Col_b, 2);
					LX_draw_text(30, 20, "Deathmatch", &FreeSans, 1, Col_a);	
			
					LX_draw_text(2, (Life_y_Pos + 12), "Leben", &TomThumb, 2, Col_white);
					LX_draw_rect(51, Life_y_Pos, 159, (Life_y_Pos + 15), Col_white, 2);
					for (int i = 1; i <= LCD_Data_REC.Life[z]; i++)
					{
						LX_fill_rect((56 + (i - 1)*(30 + 4)), (Life_y_Pos + 3), (56 + 30*i + 4*(i - 1)), (Life_y_Pos + 12), Col_red);			
					}
			
					LX_draw_text(2, (Boost_y_Pos + 12), "Speed", &TomThumb, 2, Col_white);
					LX_draw_rect(51, Boost_y_Pos, 159, (Boost_y_Pos + 15), Col_white, 2);
					LX_fill_rect(56, (Boost_y_Pos + 3), (56 + LCD_Data_REC.Modeval), (Boost_y_Pos + 12), (Col_orange + (LCD_Data_REC.Modeval*5)));		
				break;
			
				default:
				break;
			}
		
			//Aktualisieren der 4 Endgerte
			LX_start_full_refresh(&Display_Handler[z]);
			vTaskDelay(xDelay30ms);
			LX_stop_full_refresh(&Display_Handler[z]);
		}
	}		
}

void Queue_task(void*pvParam)
{

	LCD_Trans_t Test_Data =
	{
		.Mode = 0x01, 
		.Modeval = 50,
		.Life[0] = 1,
		.Life[1] = 2,
		.Life[2] = 3,
		.Life[3] = 0
	};
	
	while (1)
	{
		if (Test_Data.Modeval < 100)
		{
			Test_Data.Modeval++;
		}
		else
		{
			Test_Data.Modeval = 0;
		}
	
		xQueueSend(LCD_Queue_handler, (void*)&Test_Data, NULL);
		
		vTaskDelay(xDelay30ms);
	}
	
}


void app_main()
{	
	
	//Queue
	LCD_Queue_handler = xQueueCreate(6, sizeof(LCD_Trans_t));
	
	//Task erstellen
	xTaskCreate(lcd_task, "lcd_task", 2048, NULL, 5, NULL);
	xTaskCreate(Queue_task, "Queue_task", 2048, NULL, 5, NULL);
	
}

